@@ -635,14 +635,20 @@ class Constraints:
         """
         Construct a constraint matrix in sparse format.
 
-        Missing values, i.e. -1 in labels and vars, are ignored filtered out.
+        Missing values, i.e. -1 in labels and vars, are ignored filtered
+        out.
 
-        If filter_missings is set to True, the index of the rows and columns
-        correspond to the constraint and variable labels stored in the model.
+        If filter_missings is set to True, the index of the rows and
+        columns
+        correspond to the constraint and variable labels stored in the
+        model.
         If set to False, the rows correspond to the constraints given by
-        `m.constraints.ravel('labels', filter_missings=True)` and columns to
-        `m.variables.ravel('labels', filter_missings=True)` where `m` is the
-        underlying model. The matrix has then a shape of (`m.ncons`, `m.nvars`).
+        `m.constraints.ravel('labels', filter_missings=True)` and
+        columns to
+        `m.variables.ravel('labels', filter_missings=True)` where `m` is
+        the
+        underlying model. The matrix has then a shape of (`m.ncons`,
+        `m.nvars`).
         """
         self.sanitize_missings()
 
