@@ -10,7 +10,7 @@ import numpy as np
 import pandas as pd
 import xarray as xr
 
-from linopy import Model
+from linopy import EQUAL, GREATER_EQUAL, Model
 
 
 def test_basic_matrices():
@@ -21,7 +21,7 @@ def test_basic_matrices():
     x = m.add_variables(lower, upper, name="x")
     y = m.add_variables(name="y")
 
-    m.add_constraints(1 * x + 10 * y, "=", 0)
+    m.add_constraints(1 * x + 10 * y, EQUAL, 0)
 
     obj = (10 * x + 5 * y).sum()
     m.add_objective(obj)
@@ -40,9 +40,9 @@ def test_basic_matrices_masked():
     mask = pd.Series([True] * 8 + [False, False])
     y = m.add_variables(lower, name="y", mask=mask)
 
-    m.add_constraints(x + y, ">=", 10)
+    m.add_constraints(x + y, GREATER_EQUAL, 10)
 
-    m.add_constraints(y, ">=", 0)
+    m.add_constraints(y, GREATER_EQUAL, 0)
 
     m.add_objective(2 * x + y)
 
@@ -50,3 +50,16 @@ def test_basic_matrices_masked():
     assert m.matrices.clabels.shape == m.matrices.sense.shape
     assert m.matrices.vlabels.shape == m.matrices.ub.shape
     assert m.matrices.vlabels.shape == m.matrices.lb.shape
+
+
+def test_matrices_duplicated_variables():
+    m = Model()
+
+    x = m.add_variables(pd.Series([0, 0]), 1, name="x")
+    y = m.add_variables(4, pd.Series([8, 10]), name="y")
+    z = m.add_variables(0, pd.DataFrame([[1, 2], [3, 4], [5, 6]]).T, name="z")
+    m.add_constraints(x + x + y + y + z + z == 0)
+
+    A = m.matrices.A.todense()
+    assert A[0, 0] == 2
+    assert np.isin(np.unique(A), [0, 2]).all()
